import { createId } from '@paralleldrive/cuid2';
import { createFloatingActions, } from 'svelte-floating-ui';
import { flip } from 'svelte-floating-ui/dom';
import { get } from 'svelte/store';
export function isElementInPopover(srcElement) {
    if (!srcElement)
        return false;
    return !!srcElement.closest('[data-popover-id]');
}
const defaultFloatingUiOptions = {
    strategy: 'absolute',
    placement: 'bottom-start',
    middleware: [flip()],
};
export function createPopover(isOpen, options) {
    const id = createId();
    function customTrigger(node) {
        function triggerClick(e) {
            isOpen.update((value) => !value);
        }
        function clickOutside(e) {
            const clickedNode = e.target.closest(`[data-popover-id="${id}"]`);
            let clickedNodeId;
            if (clickedNode) {
                clickedNodeId = clickedNode.dataset.popoverId;
            }
            if (node instanceof HTMLElement && clickedNodeId !== id && !e.defaultPrevented && get(isOpen)) {
                isOpen.set(false);
            }
        }
        if (node instanceof HTMLElement) {
            node.dataset.popoverId = id;
            node.addEventListener('mouseup', triggerClick);
        }
        document.addEventListener('mouseup', clickOutside);
        return {
            destroy: () => {
                if (node instanceof HTMLElement) {
                    node.removeEventListener('mouseup', triggerClick);
                }
                document.removeEventListener('mouseup', clickOutside);
            },
        };
    }
    function customContent(node) {
        node.dataset.popoverId = id;
        return {
            destroy: () => { },
        };
    }
    const [floatingTrigger, floatingContent, update] = createFloatingActions({
        ...defaultFloatingUiOptions,
        ...options?.floatingUi,
    });
    const triggerActions = [customTrigger, floatingTrigger];
    function triggerAction(node) {
        const destructors = triggerActions.map((action) => action(node));
        return {
            destroy() {
                destructors.forEach((destructor) => {
                    if (typeof destructor?.destroy === 'function')
                        destructor.destroy();
                });
            },
        };
    }
    const contentActions = [floatingContent, customContent];
    function contentAction(node) {
        contentActions.map((action) => action(node));
        return {
            destroy() { },
        };
    }
    return [triggerAction, contentAction, update];
}
